#include <string.h>
#include <malloc.h>

#include "lua.h"
#include "lauxlib.h"
#include "lualib.h"

#include "lephisto.h"
#include "log.h"
#include "hook.h"
#include "mission.h"

// Current player missions.
Mission player_missions[MISSION_MAX];

// Mission stack.
static Mission* mission_stack = NULL; // Unmuteable after creation.
static int mission_nstack = 0;

// Extern.
extern int misn_run(Mission* misn, char* func);

// Static.
static void mission_cleanup(Mission* misn);
static void mission_free(MissionData* mission);

// Create a mission.
int mission_create(MissionData* misn) {
  int i;

  // Find last mission.
  for(i = 0; i < MISSION_MAX; i++)
    if(player_missions[i].data == NULL) break;

  // No missions left.
  if(i >= MISSION_MAX) return -1;

  player_missions[i].data = misn;

  // Init lua.
  player_missions[i].L = luaL_newstate();
  luaopen_string(player_missions[i].L); // String.format can be useful..
  misn_loadLibs(player_missions[i].L);  // Load our custom libraries.

  return 0;
}

// Clean up a mission.
static void mission_cleanup(Mission* misn) {
  hook_rmParent(misn->id); // Remove existing hooks.
  misn->data = NULL;
  if(misn->title) free(misn->title);
  if(misn->desc)  free(misn->desc);
  if(misn->reward) free(misn->reward);
  lua_close(misn->L);
}

// Free a mission.
static void mission_free(MissionData* mission) {
  if(mission->name) {
    free(mission->name);
    mission->name = NULL;
  }
  if(mission->avail.planet) {
    free(mission->avail.planet);
    mission->avail.planet = NULL;
  }
  if(mission->avail.system) {
    free(mission->avail.system);
    mission->avail.system = NULL;
  }
  if(mission->avail.factions) {
    free(mission->avail.factions);
    mission->avail.factions = NULL;
    mission->avail.nfactions = 0;
  }
}

// Load/Free.
int missions_load(void) {
#if 0
  yaml_parser_t parser;
  yaml_event_t input_event;

  memset(&parser, 0, sizeof(parser));
  memset(&input_event, 0, sizeof(input_event));

  // Set up parser.
  if(!yaml_parser_initialize(&parser)) {
    ERR("Could not initialize the parser object");
    return -1;
  }

  // Cleanup parser.
  yaml_event_delete(&input_event);
  yaml_parser_delete(&parser);
#endif

  return 0;
}

void missions_free(void) {
  int i;

  // Free the mission data.
  for(i = 0; i < mission_nstack; i++)
    mission_free(&mission_stack[i]);
  free(mission_stack);
  mission_stack = NULL;
  mission_nstack = 0;
}

void missions_cleanup(void) {
  int i;

  for(i = 0; i < MISSION_MAX; i++)
    mission_cleanup(&player_missions[i]);
}

